@@ -16,10 +16,8 @@ module Rack
     # retaining the old key for backwards compatibility
     ENV_KEY     = 'rack.cors'.freeze
 
-    OPTIONS      = 'OPTIONS'.freeze
-    VARY         = 'Vary'.freeze
-    CONTENT_TYPE = 'Content-Type'.freeze
-    TEXT_PLAIN   = 'text/plain'.freeze
+    OPTIONS     = 'OPTIONS'.freeze
+    VARY        = 'Vary'.freeze
 
     DEFAULT_VARY_HEADERS = ['Origin'].freeze
 
@@ -66,24 +64,29 @@ module Rack
     def call(env)
       env[HTTP_ORIGIN] ||= env[HTTP_X_ORIGIN] if env[HTTP_X_ORIGIN]
 
+      path = evaluate_path(env)
+
       add_headers = nil
       if env[HTTP_ORIGIN]
         debug(env) do
           [ 'Incoming Headers:',
             "  Origin: #{env[HTTP_ORIGIN]}",
+            "  Path-Info: #{path}",
             "  Access-Control-Request-Method: #{env[HTTP_ACCESS_CONTROL_REQUEST_METHOD]}",
             "  Access-Control-Request-Headers: #{env[HTTP_ACCESS_CONTROL_REQUEST_HEADERS]}"
             ].join("\n")
         end
-        if env[REQUEST_METHOD] == OPTIONS and env[HTTP_ACCESS_CONTROL_REQUEST_METHOD]
-          headers = process_preflight(env)
+
+        if env[REQUEST_METHOD] == OPTIONS && env[HTTP_ACCESS_CONTROL_REQUEST_METHOD]
+          return [400, {}, []] unless Rack::Utils.valid_path?(path)
+          headers = process_preflight(env, path)
           debug(env) do
             "Preflight Headers:\n" +
                 headers.collect{|kv| "  #{kv.join(': ')}"}.join("\n")
           end
           return [200, headers, []]
         else
-          add_headers = process_cors(env)
+          add_headers = process_cors(env, path)
         end
       else
         Result.miss(env, Result::MISS_NO_ORIGIN)
@@ -92,7 +95,7 @@ module Rack
       # This call must be done BEFORE calling the app because for some reason
       # env[PATH_INFO] gets changed after that and it won't match. (At least
       # in rails 4.1.6)
-      vary_resource = resource_for_path(env[PATH_INFO])
+      vary_resource = resource_for_path(path)
 
       status, headers, body = @app.call env
 
@@ -117,7 +120,7 @@ module Rack
         else
           DEFAULT_VARY_HEADERS
         end
-        headers[VARY] = ((vary ? vary.split(/,\s*/) : []) + cors_vary_headers).uniq.join(', ')
+        headers[VARY] = ((vary ? ([vary].flatten.map { |v| v.split(/,\s*/) }.flatten) : []) + cors_vary_headers).uniq.join(', ')
       end
 
       if debug? && result = env[RACK_CORS]
@@ -149,14 +152,28 @@ module Rack
         end
       end
 
+      def evaluate_path(env)
+        path = env[PATH_INFO]
+
+        if path
+          path = Rack::Utils.unescape_path(path)
+
+          if Rack::Utils.valid_path?(path)
+            path = Rack::Utils.clean_path_info(path)
+          end
+        end
+
+        path
+      end
+
       def all_resources
         @all_resources ||= []
       end
 
-      def process_preflight(env)
+      def process_preflight(env, path)
         result = Result.preflight(env)
 
-        resource, error = match_resource(env)
+        resource, error = match_resource(path, env)
         unless resource
           result.miss(error)
           return {}
@@ -165,8 +182,8 @@ module Rack
         return resource.process_preflight(env, result)
       end
 
-      def process_cors(env)
-        resource, error = match_resource(env)
+      def process_cors(env, path)
+        resource, error = match_resource(path, env)
         if resource
           Result.hit(env)
           cors = resource.to_headers(env)
@@ -187,8 +204,7 @@ module Rack
         nil
       end
 
-      def match_resource(env)
-        path   = env[PATH_INFO]
+      def match_resource(path, env)
         origin = env[HTTP_ORIGIN]
 
         origin_matched = false
@@ -332,7 +348,7 @@ module Rack
 
           self.path         = path
           self.credentials  = public_resource ? false : (opts[:credentials] == true)
-          self.max_age      = opts[:max_age] || 1728000
+          self.max_age      = opts[:max_age] || 7200
           self.pattern      = compile(path)
           self.if_proc      = opts[:if]
           self.vary_headers = opts[:vary] && [opts[:vary]].flatten
@@ -363,7 +379,7 @@ module Rack
         end
 
         def process_preflight(env, result)
-          headers = {CONTENT_TYPE => TEXT_PLAIN}
+          headers = {}
 
           request_method = env[HTTP_ACCESS_CONTROL_REQUEST_METHOD]
           if request_method.nil?
@@ -431,8 +447,10 @@ module Rack
             if path.respond_to? :to_str
               special_chars = %w{. + ( )}
               pattern =
-                path.to_str.gsub(/((:\w+)|[\*#{special_chars.join}])/) do |match|
+                path.to_str.gsub(/((:\w+)|\/\*|[\*#{special_chars.join}])/) do |match|
                   case match
+                  when "/*"
+                    "\\/?(.*?)"
                   when "*"
                     "(.*?)"
                   when *special_chars
